@@ -1,22 +1,21 @@
-"""
-Copyright 2012 Dian-Je Tsai and Wantoto Inc
 
-Licensed under the Apache License, Version 2.0 (the "License");
-you may not use this file except in compliance with the License.
-You may obtain a copy of the License at
+# Copyright 2012 Dian-Je Tsai and Wantoto Inc
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 
-    http://www.apache.org/licenses/LICENSE-2.0
-
-Unless required by applicable law or agreed to in writing, software
-distributed under the License is distributed on an "AS IS" BASIS,
-WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-See the License for the specific language governing permissions and
-limitations under the License.
-"""
 import datetime
 import sys
 import os
-import re
 from fabric.api import (hide, local, env, execute, abort, lcd, sudo, run, cd, puts, settings as fab_settings)
 from fabric.colors import red, green, yellow, cyan
 from fabric.contrib import django as fab_django
@@ -60,14 +59,17 @@ def deploy_setting(project_name=None, deploy_settings_path=None, **setting_patch
     # Find deploy_settings
     if deploy_settings_path is None:
         deploy_settings_path = getattr(env, 'deploy_settings', None)
+
     if deploy_settings_path is None:
         deploy_settings_path = os.path.abspath(os.path.join(os.getcwd(), 'deploy_settings.py'))
         if not os.path.exists(deploy_settings_path):
             deploy_settings_path = None
+
     if deploy_settings_path is None and project_name is not None:
         deploy_settings_path = os.path.abspath(os.path.join(os.getcwd(), project_name, 'deploy_settings.py'))
         if not os.path.exists(deploy_settings_path):
             deploy_settings_path = None
+
     if deploy_settings_path is None:
         print error(red('Cannot find deploy settings', bold=True))
 
@@ -93,8 +95,7 @@ def deploy_setting(project_name=None, deploy_settings_path=None, **setting_patch
 
 
 @task(default=True)
-def deploy(project_name=None, deploy_settings_path=None, commit_msg='deploy',
-           verbose=0, update_apache_conf=False, **additional_settings):
+def deploy(project_name=None, deploy_settings_path=None, commit_msg='deploy', **additional_settings):
     """Deploy current django project to targeted web servers
 
     This is a public task. It will prepare arguments for the
@@ -102,19 +103,21 @@ def deploy(project_name=None, deploy_settings_path=None, commit_msg='deploy',
 
     :param project_name:
         The name of project. If you don't specify this, eggplant will find in following locations until found:
+
         1. 'env' of current fabric context (env.project_name = <PROJECT_NAME>)
         2. 'deploy_settings.py' (If "deploy_settings.py" can be found without "project_name")
+
     :param deploy_settings_path:
         The location of "deploy_settings.py". If you don't specify this,
         eggplant will find in following ways until found:
+
         1. 'env' of current fabric context (env.deploy_settings = <DEPLOY_SETTING_MODULE>)
         2. <CURRENT_WORKING_DIRECTORY>/deploy_settings.py
         3. <CURRENT_WORKING_DIRECTORY>/<PROJECT_NAME>/deploy_settings.py
             (If 'project_name' is specified or can be found without deploy_settings)
+
     :param commit_msg:
         The message used for git commit in deploy target
-    :param verbose:
-        Print information or not. Default is 0. Levels=[0, 1, 2, 3]
     :param update_apache_conf:
         Update Apache configuration file or not.
         If this is first-time deploy, the value will be forced to True.
@@ -128,10 +131,10 @@ def deploy(project_name=None, deploy_settings_path=None, commit_msg='deploy',
     )
     project_name = env.project_name
     deploy_settings = env.deploy_settings
-    env.update_apache_conf = update_apache_conf
+    env.update_apache_conf = deploy_settings.get('UPDATE_APACHE_CONF')
 
     # Check output level
-    env.verbose = verbose = int(verbose)
+    env.verbose = verbose = deploy_settings.get('VERBOSE')
     if verbose == 0:
         hide_output_status = ['everything']
     elif verbose == 1:
@@ -159,10 +162,19 @@ def deploy(project_name=None, deploy_settings_path=None, commit_msg='deploy',
         env.tmp_directory = tmp_directory
         # Copy codebase
         puts(cyan('Copy your codebase to temp directory'))
-        original_src_root = deploy_settings.get('SOURCE_ROOT', os.getcwd())
+        original_src_root = deploy_settings.get('SOURCE_ROOT')
+
+        # Project settings module
+        project_settings_module = deploy_settings.get('PROJECT_SETTINGS')
+        env.project_settings_path = project_settings_path = project_settings_module.replace('.', '/') + '.py'
+        # Use symbolic link or not?
+        use_symbolic_link_settings = deploy_settings.get(
+            'USE_SYMBOLIC_LINK_SETTINGS', os.path.islink(project_settings_path)
+        )
+
         # Rsync
-        copy_codebase_rsync_args = '-a --delete --exclude .git'
-        ignore_file_list = deploy_settings.get('IGNORE_FILE_LIST', os.path.join(original_src_root, '.gitignore'))
+        copy_codebase_rsync_args = '-DLprtza --delete --exclude .git'
+        ignore_file_list = os.path.abspath(deploy_settings.get('IGNORE_FILE_LIST'))
         if ignore_file_list is not None and os.path.exists(ignore_file_list):
             copy_codebase_rsync_args += ' --exclude-from \'%s\'' % os.path.relpath(ignore_file_list, os.getcwd())
         if not original_src_root.endswith('/'):
@@ -170,6 +182,17 @@ def deploy(project_name=None, deploy_settings_path=None, commit_msg='deploy',
         local('rsync %s %s %s' % (
             copy_codebase_rsync_args, original_src_root + ('' if original_src_root.endswith('/') else '/'),
             tmp_directory))
+        # Replace original git-ignore ...
+        # Original git-ignore is used for SCM repository.
+        # Now we need here is for remote base, so we copied/replaced the SCM one with IGNORE_FILE_LIST
+        if ignore_file_list is not None and os.path.exists(ignore_file_list):
+            target_gitignore = '%s/.gitignore' % tmp_directory
+            # Copy only if they are different
+            with fab_settings(hide('everything'), warn_only=True):
+                diff_result = local('diff %s %s' % (ignore_file_list, target_gitignore))
+                if diff_result.return_code == 1:
+                    local(rsync(ignore_file_list, target_gitignore))
+
         # Set project source root and append to PYTHONPATH
         env.project_src_root = project_src_root = tmp_directory
         sys.path = [project_src_root] + sys.path
@@ -178,18 +201,10 @@ def deploy(project_name=None, deploy_settings_path=None, commit_msg='deploy',
 
         with lcd(tmp_directory):
             # Switch settings
-            project_settings_module = deploy_settings.get('PROJECT_SETTINGS', '%s.settings' % project_name)
-            env.project_settings_path = project_settings_path = project_settings_module.replace('.', '/') + '.py'
-            # Use symbolic link or not?
-            use_symbolic_link_settings = deploy_settings.get(
-                'USE_SYMBOLIC_LINK_SETTINGS', os.path.islink(project_settings_path)
-            )
             if use_symbolic_link_settings:
                 # Relink to production one
                 puts(cyan('Move to production settings'))
-                production_settings_module = deploy_settings.get(
-                    'PRODUCTION_SETTINGS', '%s.production_settings' % project_name
-                )
+                production_settings_module = deploy_settings.get('PRODUCTION_SETTINGS')
                 production_settings_path = production_settings_module.replace('.', '/') + '.py'
                 local('rm -rf %s' % project_settings_path)
                 local('ln -s %s %s' % (
@@ -201,8 +216,7 @@ def deploy(project_name=None, deploy_settings_path=None, commit_msg='deploy',
             fab_django.settings_module(project_settings_module)
             from django.conf import settings as django_settings
             from django.core.management import get_commands
-
-            env.django_settings = django_settings
+            env.django_settings = django_settings  #TODO: load which one?
 
             # Set paths
             remote_base_path = deploy_settings.get('REMOTE_BASE')
@@ -280,12 +294,11 @@ def deploy(project_name=None, deploy_settings_path=None, commit_msg='deploy',
 
             # Replace gitignore. We wanna preserve pyc
             project_src_root_gitignore = os.path.join(project_src_root, '.gitignore')
-            pyc_pattern = re.compile(r'.*\.py[c|o]$')
             gitignores = []
             with open(project_src_root_gitignore, 'r') as f:
                 raw_gitignores = f.read().strip().split('\n')
             for gitignore_item in raw_gitignores:
-                if pyc_pattern.match(gitignore_item) is None:
+                if gitignore_item not in ('*.pyc', '*.pyo', '*.py[co]'):
                     gitignores += [gitignore_item]
             gitignores += deploy_settings.get('ADDITIONAL_IGNORE_LIST')
             gitignores += [
@@ -339,7 +352,7 @@ def deploy_to_remote():
     source_root_name = 'source'
     venv_name = 'venv'
     log_name = 'logs'
-    remote_project_path = os.path.join(remote_base_path, project_name)
+    remote_project_path = os.path.join(remote_base_path, deploy_settings.get('REMOTE_PROJECT_NAME'))
     remote_project_source_root = env.remote_project_source_root = os.path.join(remote_project_path, source_root_name)
     remote_project_venv = env.remote_project_venv = os.path.join(remote_project_path, venv_name)
     remote_project_log_root = env.remote_project_log_root = os.path.join(remote_project_path, log_name)
@@ -401,7 +414,7 @@ def deploy_to_remote():
             target_run('git init')
             target_run('echo "# ignore files" >> .gitignore')
             target_run('git add .gitignore && git commit -m "Init commit"')
-            # Virtualenv
+        # Virtualenv
         puts(yellow('First time deploy, initialize virtualenv:'))
         with target_cd(remote_project_path):
             target_run('virtualenv %s' % venv_name)
@@ -415,20 +428,15 @@ def deploy_to_remote():
     if deploy_settings.get('INSTALL_APACHE_CONF') and update_apache_conf:
         puts(cyan('Generate configuration file of mod_wsgi and Apache'))
         # Get Apache conf template
-        apache_conf_template_filename = 'apache.conf.jinja2'
-        apache_conf_template_path = deploy_settings.get('APACHE_CONF_TEMPLATE')
-        if apache_conf_template_path is None:
-            apache_conf_template_path = os.path.join(
-                project_src_root,
-                os.path.dirname(project_settings_path),
-                apache_conf_template_filename
-            )
+        apache_conf_template_filename = deploy_settings.get('APACHE_CONF_TEMPLATE_FILENAME')
+        apache_conf_template_path = os.path.join(
+            project_src_root,
+            os.path.dirname(project_settings_path),
+            apache_conf_template_filename
+        )
         if not os.path.exists(apache_conf_template_path):
-            apache_conf_template_path = os.path.join(
-                os.path.dirname(__file__),
-                'deploy/%s' % apache_conf_template_filename
-            )
-            # Make context for apache conf
+            apache_conf_template_path = deploy_settings.get('APACHE_CONF_TEMPLATE_PATH')
+        # Make context for apache conf
         with hide('everything'):
             raw_python_version = target_run('python -c "import sys; print sys.version_info"', **capture_output)
             python_version = map(lambda x: x.strip().replace('\'', ''), raw_python_version[1:-1].split(','))
@@ -450,7 +458,7 @@ def deploy_to_remote():
             ('URL_PREFIX',),
             ('VIRTUALENV_SITE_PACKAGES_PATH', default_site_packages),
             ('WSGI_PROCESS_GROUP',),
-            ('WSGI_PROCESS_NAME', project_name),
+            ('WSGI_PROCESS_NAME',),
             ('WSGI_PROCESS_USER',),
             ('WSGI_PROCESSES_COUNT',),
             ('WSGI_PYTHON_OPTIMIZE_LEVEL',),
@@ -482,7 +490,7 @@ def deploy_to_remote():
         rsync_argument += ' -v'
     local(rsync(
         project_src_root + ('' if project_src_root.endswith('/') else '/'),
-        (host_string + ':' if is_remote else '') + remote_project_source_root, rsync_schema=rsync_argument))
+        (host_string + ':' if is_remote else '') + remote_project_source_root, rsync_argument=rsync_argument))
 
     # Setup Remote
     if deploy_settings.get('RUN_REMOTE_SETUP'):
@@ -532,7 +540,7 @@ def deploy_to_remote():
     if deploy_settings.get('INSTALL_APACHE_CONF') and update_apache_conf:
         print(cyan('Install apache conf'))
         apache_conf_path = deploy_settings.get('APACHE_CONF_DIR')
-        apache_conf_target = os.path.join(apache_conf_path, '%s.conf' % project_name)
+        apache_conf_target = os.path.join(apache_conf_path, '%s.conf' % deploy_settings.get('APACHE_CONF_FILE'))
         print cyan('Installing Apache conf')
         with target_cd(remote_project_source_root):
             target_sudo('mv %s %s' % (apache_conf_tmp_name, apache_conf_target))
