@@ -29,15 +29,18 @@ https://github.com/derek-schaefer/django-json-field/
 
 """
 
-import json
+from eggplant.forms.fields.json_field import JSONFormField
+
 from django.db import models
-#from django.utils import simplejson as json
+from django.utils import simplejson as json
+from django.core import exceptions
 from django.core.serializers.json import DjangoJSONEncoder
 from django.utils.translation import ugettext as _
 from django.core.exceptions import ImproperlyConfigured
-from eggplant.forms.fields.json_field import JSONFormField
 
 import re
+from decimal import Decimal
+from datetime import datetime
 try:
     from dateutil import parser as date_parser
 except ImportError:
@@ -111,7 +114,7 @@ class JSONField(models.TextField):
         if not encoder_kwargs and encoder:
             encoder_kwargs.update({'cls':encoder})
         if not decoder_kwargs and decoder:
-            decoder_kwargs.update({'cls':decoder})
+            decoder_kwargs.update({'cls':decoder, 'parse_float':Decimal})
         self.encoder_kwargs = encoder_kwargs
         self.decoder_kwargs = decoder_kwargs
 
